// HTMLView.cpp : implementation file
//

#include "stdafx.h"
#include "ITBReader.h"

#include "HTMLView.h"
#include "HTMLParser.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Bottom offset defines
#define BO_PARAGRAPH		10
#define SELECTION_OFFSET	2
#define HIGHLIGHT_OFFSET	2

/////////////////////////////////////////////////////////////////////////////
// CHTMLView
bool CHTMLView::s_bIsClassRegistered = false;

IMPLEMENT_DYNCREATE(CHTMLView, CView)

CHTMLView::CHTMLView() : m_rcMargins(20, 20, 20, 20)
{
	m_currPos.charPos.nIndex = 0;
	m_currPos.charPos.pTokenInfo = NULL;
	m_currPos.pLine = NULL;
	m_currPos.bIsLineEnd = false;
	
	m_caretPos.x = 20;
	m_caretPos.y = 20;
	m_nCaretHeight = 15;	

	m_bTextSelecting = false;
	m_bTextSelected = false;

	m_selStart.nIndex = 0;
	m_selStart.pTokenInfo = NULL;
	m_selEnd = m_selStart;
	m_rcSelection.SetRect(0, 0, 0 ,0);

	m_bModified = false;
	/*m_pdcMemory = new CDC;
	m_pScreenBufferBmp = new CBitmap;*/
}

CHTMLView::~CHTMLView()
{
	/*delete m_pScreenBufferBmp;
	delete m_pdcMemory;*/
}


BEGIN_MESSAGE_MAP(CHTMLView, CView)
	//{{AFX_MSG_MAP(CHTMLView)
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_VSCROLL()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_COMMAND(ID_HIGHLIGHT_TEXT, OnHighlightText)
	ON_UPDATE_COMMAND_UI(ID_HIGHLIGHT_TEXT, OnUpdateHighlightText)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHTMLView drawing

void CHTMLView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	/*if (m_pdcMemory->GetSafeHdc() == NULL)
	{
		CClienteDC dc(this);
		OnPrepareDC(&dc);
		m_pdcMemory->CreateCompatibleDC(&dc);
	}*/
}

int CHTMLView::GetAbsCharPos(const CHARPOS& cp)
{
	CHARPOS cpFound;
	int i;
	
	cpFound.pTokenInfo = m_tokens.GetHeadPosition();
	cpFound.nIndex = 0;
	
	for (i = 0; cpFound != cp; i++)
		cpFound = GetNextCharPos(cpFound);

	return i; 
}

CHARPOS CHTMLView::GetCharPos(int nAbsIndex)
{
	CHARPOS cp;
	
	cp.pTokenInfo = m_tokens.GetHeadPosition();
	cp.nIndex = 0;
	
	for (int i = 1; i <= nAbsIndex; i++)
		cp = GetNextCharPos(cp);

	return cp;
}

RELCHARPOS CHTMLView::GetRelCharPos(const CHARPOS& cp)
{
	RELCHARPOS rcp;
	TEXTLINE* pLine;
	//rcp.charPos = cp;

	POSITION linePos = m_page.GetHeadPosition();

	while(linePos)
	{
		rcp.pLine = linePos;	// Stored now thou just in case it is.
		pLine = &m_page.GetNext(linePos);

		rcp.charPos = pLine->first;

		while(rcp.charPos != cp && rcp.charPos != pLine->last)
			rcp.charPos = GetNextCharPos(rcp.charPos);

		if (rcp.charPos == cp)
			return rcp;			// rcp.pLine param was previously stored.
	}
	
	// It must not get here. Always it should find a line pos.
	ASSERT(false);
	rcp.pLine = NULL;
	return rcp;
}

void CHTMLView::DrawHighlightedFrames(CDC* pDC)
{
	/*CRect rect;
	
	GetClientRect(rect);
	rect.left += m_rcMargins.left;

	int x = rect.left;
	int y = rect.top + m_rcMargins.top;

	POSITION linePos = m_page.FindIndex(GetScrollPos(SB_VERT));
	TEXTLINE line;
	CHARPOS cp;

	COLORREF oldColor = pDC->GetBkColor();
	int i = 0, j, cx = 0;
	while (linePos && y <= rect.bottom) 
	{
		line = m_page.GetNext(linePos);

		cp = line.first;

		for(j=0; j < 6; j++)
			x += GetNextWidth(&cp);
		
		for(j=0, cx = x; j < 6; j++)
			cx += GetNextWidth(&cp);
		
		if (i++ == 2)
			pDC->FillSolidRect(x, y, cx - x, line.nHeight + 2, RGB(255, 255, 0));

		y += line.TotalHeight();
		x = rect.left;
	}

	pDC->SetBkColor(oldColor);*/

	POSITION pos = m_hlList.GetHeadPosition();

	while (pos)
		DrawSingleHighlightedFrame(m_hlList.GetNext(pos), pDC);
}

void CHTMLView::DrawSingleHighlightedFrame(const HIGHLIGHT& hl, CDC* pDC)
{
	RELCHARPOS selStart, selEnd;
	CRect rcClient, rcFrame;
	COLORREF oldColor = pDC->GetBkColor();

	selStart = GetRelCharPos(hl.start);
	selEnd = GetRelCharPos(hl.end);

	GetClientRect(rcClient);
	rcClient.left += m_rcMargins.left;

	rcFrame.left = rcClient.left;
	rcFrame.top = rcClient.top + m_rcMargins.top;

	POSITION curLinePos, linePos = m_page.FindIndex(GetScrollPos(SB_VERT));
	TEXTLINE line;
	CHARPOS cp;
	bool bDrawSel = false;

	while (linePos && rcFrame.top <= rcClient.bottom) 
	{
		curLinePos = linePos;
		line = m_page.GetNext(linePos);
		cp = line.first;

		if (curLinePos == selStart.pLine)
		{
			bDrawSel = true;
			
			while (selStart.charPos != cp)
				rcFrame.left += GetNextWidth(&cp);
		}

		if (bDrawSel)
		{
			rcFrame.right = rcFrame.left;

			do
			{
				rcFrame.right += GetNextWidth(&cp);
			
				if (cp != selEnd.charPos && IsLineLastChar(line, cp))
				{
					rcFrame.right += GetNextWidth(&cp);
					break;
				}
			} while (cp != selEnd.charPos);

			rcFrame.bottom = rcFrame.top + line.nHeight + HIGHLIGHT_OFFSET;
			pDC->FillSolidRect(rcFrame, hl.color);
		}
		
		if (curLinePos == selEnd.pLine)
			break; // All done

		rcFrame.top += line.TotalHeight();
		rcFrame.left = rcClient.left;
	}

	pDC->SetBkColor(oldColor);
}

void CHTMLView::DrawSelection(CDC* pDC)
{
	if (!m_bTextSelected || m_selStart == m_selEnd)
		return;	// Nothing to do
	
	RELCHARPOS selStart, selEnd;
	CRect rcClient, rcSel;

	selStart = GetRelCharPos(m_selStart);
	selEnd = GetRelCharPos(m_selEnd);

	// Invert params to represent the correct order if needed
	if (!IsSelectionInOrder())
	{
		RELCHARPOS aux = selStart;
		selStart = selEnd;
		selEnd = aux;
	}

	GetClientRect(rcClient);
	rcClient.left += m_rcMargins.left;

	rcSel.left = rcClient.left;
	rcSel.top = rcClient.top + m_rcMargins.top;

	POSITION curLinePos, linePos = m_page.FindIndex(GetScrollPos(SB_VERT));
	TEXTLINE line;
	CHARPOS cp;
	bool bDrawSel = false;

	m_rcSelection.SetRect(0, 0, 0 ,0);

	while (linePos && rcSel.top <= rcClient.bottom) 
	{
		curLinePos = linePos;
		line = m_page.GetNext(linePos);
		cp = line.first;

		if (curLinePos == selStart.pLine)
		{
			bDrawSel = true;
			
			while (selStart.charPos != cp)
				rcSel.left += GetNextWidth(&cp);
		}

		if (bDrawSel)
		{
			rcSel.right = rcSel.left;

			do
			{
				rcSel.right += GetNextWidth(&cp);
			
				if (cp != selEnd.charPos && IsLineLastChar(line, cp))
				{
					rcSel.right += GetNextWidth(&cp);
					break;
				}
			} while (cp != selEnd.charPos);

			rcSel.bottom = rcSel.top + line.nHeight + SELECTION_OFFSET;
			if (pDC) pDC->InvertRect(rcSel);
			m_rcSelection |= rcSel;
		}
		
		if (curLinePos == selEnd.pLine)
			break; // All done

		rcSel.top += line.TotalHeight();
		rcSel.left = rcClient.left;
	}
}

void CHTMLView::OnDraw(CDC* pDC)
{
	//CDocument* pDoc = GetDocument();
	
	////////////////////////////////////////////////////////
	// Debug
	/*TokenInfo ti;
	POSITION pos = m_tokens.GetHeadPosition();
	CString str, tok;
	int y = 10;

	while(pos) {
		ti = m_tokens.GetNext(pos);
		CString tok(ti.m_pText, ti.m_nLen);

		str.Format(_T("Token=%s, bold=%c, italic=%c"), 
			(LPCTSTR) tok, 
			(ti.m_nStyle & TI_BOLD) ? 'S':'N', 
			(ti.m_nStyle & TI_ITALIC) ? 'S':'N');
		
		pDC->ExtTextOut(10, y, 0, NULL, str, NULL);
		y += 15;
	}*/
	///////////////////////////////////////////////////////
	DrawHighlightedFrames(pDC);
	
	int nOldBkMode = pDC->SetBkMode(TRANSPARENT);
	DrawLines(pDC);
	pDC->SetBkMode(nOldBkMode);

	DrawSelection(pDC);
}

/////////////////////////////////////////////////////////////////////////////
// CHTMLView diagnostics

#ifdef _DEBUG
void CHTMLView::AssertValid() const
{
	CView::AssertValid();
}

void CHTMLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CHTMLView message handlers

void CHTMLView::DeleteContents()
{
	POSITION pos = m_tokens.GetHeadPosition();

	while(pos)
		m_tokens.GetNext(pos).Empty();

	m_tokens.RemoveAll();
	
	SetScrollPage();

	m_bTextSelected = false;
	m_hlList.RemoveAll();
}

bool CHTMLView::SetHTMLText(TCHAR* pBuffer, DWORD dwBufferSize)
{
	DeleteContents();

	CHTMLParser parser;
	TOKEN token;
	CTokenInfo ti;

	if (!parser.Open(pBuffer, dwBufferSize)) 
		return false;
	
	ti.m_pWidths = NULL;

	while (parser.GetToken(&token) != TK_EOF) 
	{
		switch (token.id)
		{
			case TK_TEXT:
				ti.SetText(token.p, true);
				m_tokens.AddTail(ti);
				ti.m_nStyle &= ~TI_NEWLINE;
				ti.m_nStyle &= ~TI_DATAOWNER;
				break;
			case TK_TAG_B: 
				ti.m_nStyle |= TI_BOLD; 
				break;
			case TK_ETAG_B: 
				ti.m_nStyle &= ~TI_BOLD;
				break;
			case TK_TAG_I: 
				ti.m_nStyle |= TI_ITALIC;
				break;
			case TK_ETAG_I: 
				ti.m_nStyle &= ~TI_ITALIC; 
				break;
			case TK_TAG_H1: 
				ti.m_nStyle |= TI_H1 | TI_NEWLINE;
				break;
			case TK_TAG_H2:
				ti.m_nStyle |= TI_H2 | TI_NEWLINE;
				break;
			case TK_TAG_H3:
				ti.m_nStyle |= TI_H3 | TI_NEWLINE;
				break;
			case TK_ETAG_H1: 
				ti.m_nStyle &= ~TI_H1;
				ti.m_nStyle |= TI_NEWLINE;
				break;
			case TK_ETAG_H2:
				ti.m_nStyle &= ~TI_H2;
				ti.m_nStyle |= TI_NEWLINE;
				break;
			case TK_ETAG_H3: 
				ti.m_nStyle &= ~TI_H3;
				ti.m_nStyle |= TI_NEWLINE;
				break;
			case TK_TAG_P:
				ti.m_nStyle |= TI_NEWLINE;
				break;
			case TK_ETAG_P:
				ti.m_nStyle |= TI_NEWLINE;
				break;
			case TK_TAG_A:
				ti.m_nStyle |= TI_HYPERLINK;
				ti.SetData(token.p + 2/*"<A"*/);
				break;
			case TK_ETAG_A:
				ti.m_nStyle &= ~TI_HYPERLINK;
				ti.m_szData = NULL;
				break;
		}
	}

	parser.Close();

	PreparePage();
	BreakLines();
	SetScrollPos(SB_VERT, 0, FALSE);
	SetScrollPage();

	// Reset current line and char position
	m_currPos.pLine = m_page.GetHeadPosition();
	m_currPos.charPos.pTokenInfo = m_tokens.GetHeadPosition();
	m_currPos.charPos.nIndex = 0;

	Invalidate();

	return true;
}
//int* pWidths
int CHTMLView::CalcLineBreak(CTokenInfo& ti, int nLen, int cx, int& x)
{
	ASSERT(ti.m_pWidths && nLen > 0 && cx >= 0);
	int i = 0, dx, ilws = -1;	// ilws = i of last white space
	x = 0;

	do {
		dx = ti.m_pWidths[i];
		if (ti.m_pText[i] == ' ') ilws = i;
		if (x + dx <= cx) {
			x += dx;
			i++;
		}
		else return (ilws == -1 || ilws >= nLen) ? i:ilws + 1;
	} while(i < nLen);
	
	return nLen;
}

void CHTMLView::PrepareFont(CFont* pFont, int nStyle)
{
	BOOL bItalic = (nStyle & TI_ITALIC) ? TRUE:FALSE;
	int nWeight = (nStyle & TI_BOLD) ? 700:400;
	BOOL bUnderline = (nStyle & TI_UNDERLINE || nStyle & TI_HYPERLINK) ? TRUE:FALSE;
	int nHeight;
	BYTE nPitchAndFamily = DEFAULT_PITCH;
	LPCTSTR lpszFacename;

	if (nStyle & TI_H1) {
		nHeight = -30;
		nPitchAndFamily |= FF_SWISS;
		lpszFacename = _T("Arial");
	}
	else if (nStyle & TI_H2) {
		nHeight = -20;
		nPitchAndFamily |= FF_SWISS;
		lpszFacename = _T("Arial");
	}
	else if (nStyle & TI_H3) {
		nHeight = -18;
		nWeight = 700;
		nPitchAndFamily |= FF_ROMAN;
		lpszFacename = _T("Times New Roman");
	}
	else {
		// TI_TEXT
		nHeight = -16;
		nPitchAndFamily |= FF_ROMAN;
		lpszFacename = _T("Times New Roman");
	}

	pFont->CreateFont(nHeight, 0, 0, 0, nWeight, bItalic, bUnderline, 
			0, ANSI_CHARSET, OUT_DEFAULT_PRECIS,
			CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,
			nPitchAndFamily, lpszFacename);
}

void CHTMLView::PreparePage()
{
	TOKENLIST& list = m_tokens;;
	POSITION pos = list.GetHeadPosition();
	CDC* pDC = GetDC();
	CFont* pOldFont = pDC->GetCurrentFont();
	CTokenInfo* pTI;
	CFont font;
	int i;

	// Walk by the hole token list sizing the char widths
	while (pos)
	{
		pTI = &list.GetNext(pos);

		// Make the font based on the style
		PrepareFont(&font, pTI->m_nStyle);

		// Select the font in the DC
		pDC->SelectObject(&font);

		// Alloc memory if its the first sizing
		if (pTI->m_pWidths == NULL)
			pTI->m_pWidths = new int[pTI->m_nLen];
		
		pTI->m_nHeight = pDC->GetTextExtent(&(pTI->m_pText[0]), 1).cy;
		
		// Size char widths
		for (i = 0; i < pTI->m_nLen; i++)
			pTI->m_pWidths[i] = pDC->GetTextExtent(&(pTI->m_pText[i]), 1).cx;

		// Restore previous font and delete current font
		pDC->SelectObject(pOldFont);
		font.DeleteObject();
	}

	ReleaseDC(pDC);
}

int CHTMLView::GetTopOffset(int nStyle)
{
	if (nStyle & TI_H1) return 10;
	else if (nStyle & TI_H2) return 5;
	else if (nStyle & TI_H3) return 0;
	else return 0;
}

int CHTMLView::GetBottomOffset(int nStyle)
{
	if (nStyle & TI_H1) return 20;
	else if (nStyle & TI_H2) return 10;
	else if (nStyle & TI_H3) return 5;
	else return 5;
}

void CHTMLView::BreakLines()
{
	CTokenInfo ti;
	TOKENLIST& list = m_tokens;
	POSITION nextPos = list.GetHeadPosition();
	POSITION currPos, prevPos;
	TEXTLINE line;
	int nRemLen;	// Remnant Lenght 
	int nPrevLen;	// Previous Lenght
	int x, dx, nCharCount;
	int nLastStyle = 0;
	CRect rect;
	
	GetClientRect(rect);
	rect.DeflateRect(m_rcMargins);
	
	m_page.RemoveAll();
	x = rect.left;
	line.first.pTokenInfo = nextPos;
	line.first.nIndex = 0;
	line.nHeight = 0;
	line.nBottomOffset = 0;
	line.nTopOffset = 0;

	currPos = NULL;

	while (nextPos) 
	{
		prevPos = currPos;
		currPos = nextPos;
		ti = list.GetNext(nextPos);

		// If a new line is required and it isn't the first token.
		if ((ti.m_nStyle & TI_NEWLINE) && prevPos != NULL)
		{
			line.last.pTokenInfo = prevPos;
			line.last.nIndex = nPrevLen - 1;
			line.nTopOffset = GetTopOffset(ti.m_nStyle);
			line.nBottomOffset = GetBottomOffset(nLastStyle);
			m_page.AddTail(line);
			line.first.pTokenInfo = currPos;
			line.first.nIndex = 0;
			line.nHeight = 0;
			line.nTopOffset = 0;
			line.nBottomOffset = 0;
			x = rect.left;
		}
		
		if (line.nHeight < ti.m_nHeight)
			line.nHeight = ti.m_nHeight;

		nLastStyle = ti.m_nStyle;

		for (nRemLen = ti.m_nLen; nRemLen > 0; nRemLen -= nCharCount)
		{
			nCharCount = CalcLineBreak(ti, nRemLen, 
				rect.right - x, dx);

			if (nCharCount < nRemLen) 
			{
				// Avoid a possible endesly loop
				if (nCharCount == 0 && x == rect.left) {
					TRACE(_T("Too small window!!!"));
					return;
				}
				
				// Is new line and new token. Special case
				if (nRemLen == ti.m_nLen && nCharCount == 0)
				{
					line.last.pTokenInfo = prevPos;
					line.last.nIndex = nPrevLen - 1;	// The whole previous string
					m_page.AddTail(line);
					line.first.pTokenInfo = currPos;
					line.first.nIndex = 0;
					line.nHeight = ti.m_nHeight;
					line.nTopOffset = 0;
					line.nBottomOffset = 0;
				}
				else
				{
					line.last.pTokenInfo = currPos;
					line.last.nIndex = ti.m_nLen - (nRemLen - nCharCount) - 1;
					m_page.AddTail(line);
					line.first.pTokenInfo = currPos;
					line.first.nIndex = line.last.nIndex + 1;
					line.nHeight = ti.m_nHeight;
					line.nTopOffset = 0;
					line.nBottomOffset = 0;
				}

				ti.m_pWidths += nCharCount;
				
				// Reset line coordinate
				x = rect.left;
			}
			else
			{
				x += dx;
			}
		}
		// I may need this lenght later
		nPrevLen = ti.m_nLen;
	}

	// There is still one more line to add. The page last line
	if (currPos != NULL)
	{
		line.last.pTokenInfo = currPos;
		line.last.nIndex = nPrevLen - 1;
		m_page.AddTail(line);
	}
}

CHARPOS CHTMLView::GetNextCharPos(CHARPOS charPos)
{
	const CTokenInfo& ti = m_tokens.GetAt(charPos.pTokenInfo);
	
	ASSERT (charPos.nIndex + 1 <= ti.m_nLen);
	
	if (charPos.nIndex + 1 == ti.m_nLen) {
		m_tokens.GetNext(charPos.pTokenInfo);
		charPos.nIndex = 0;
	}
	else
		charPos.nIndex++;

	return charPos;
}

int CHTMLView::GetNextWidth(CHARPOS* pCharPos)
{
	CTokenInfo* pTI = &m_tokens.GetAt(pCharPos->pTokenInfo);
	int w = pTI->m_pWidths[pCharPos->nIndex++];
	
	if (pCharPos->nIndex >= pTI->m_nLen) {
		pCharPos->nIndex = 0;
		m_tokens.GetNext(pCharPos->pTokenInfo);
	}
	
	return w;
}

int CHTMLView::GetPrevWidth(CHARPOS* pCharPos)
{
	CTokenInfo* pTI;
	pCharPos->nIndex--;
	
	if (pCharPos->nIndex < 0) {
		if (pCharPos->pTokenInfo == NULL)
			pCharPos->pTokenInfo = m_tokens.GetTailPosition();
		else
			m_tokens.GetPrev(pCharPos->pTokenInfo);

		pTI = &m_tokens.GetAt(pCharPos->pTokenInfo);
		pCharPos->nIndex = pTI->m_nLen - 1;
	}	
	else
		pTI = &m_tokens.GetAt(pCharPos->pTokenInfo);
	
	return pTI->m_pWidths[pCharPos->nIndex];
}

bool CHTMLView::IsLineFirstChar(const TEXTLINE& line, const CHARPOS& cp)
{
	return (line.first.pTokenInfo == cp.pTokenInfo &&
			line.first.nIndex == cp.nIndex);
}

bool CHTMLView::IsLineLastChar(const TEXTLINE& line, const CHARPOS& cp)
{
	return (line.last.pTokenInfo == cp.pTokenInfo &&
			line.last.nIndex == cp.nIndex);
}

bool CHTMLView::IsLineEnd(const TEXTLINE& line, const CHARPOS& cp)
{
	if (line.last.pTokenInfo == cp.pTokenInfo)
	{
		if (line.last.nIndex >= cp.nIndex)
			return false;
		else
			return true;
	}
	
	POSITION nextTokenInfo = line.last.pTokenInfo;
	m_page.GetNext(nextTokenInfo);

	if (cp.pTokenInfo == nextTokenInfo &&
		cp.nIndex == 0)
		return true;

	return false;
}

bool CHTMLView::GetCharFromPt(const CPoint& pt, RELCHARPOS* pRCP, CPoint* pPt /*=NULL*/)
{
	if (m_page.IsEmpty())
		return false;
	
	ASSERT(pRCP);

	TEXTLINE line;
	int y = m_rcMargins.top;

	// Find line
	POSITION linePos = m_page.FindIndex(GetScrollPos(SB_VERT));

	do {
		pRCP->pLine = linePos;
		line = m_page.GetNext(linePos);

		if (linePos && y + line.TotalHeight() < pt.y)
			y += line.TotalHeight();
		else if (!linePos && y + line.TotalHeight() < pt.y)
			return false;
		else
			break;			

	} while (true);

	// Find character
	TOKENLIST& list = m_tokens;
	CHARPOS cp;
	CTokenInfo ti;
	int x = m_rcMargins.left, dx;

	cp = pRCP->charPos = line.first;
	
	while (!IsLineLastChar(line, cp))
	{
		dx = GetNextWidth(&cp);

		if (x + dx >= pt.x)
		{
			if (pt.x - x >= x + dx - pt.x)
			{
				x += dx;
				pRCP->charPos = cp;
			}
			
			if (pPt)
			{
				pPt->x = x;
				pPt->y = y;
			}

			return true;
		}
		else
		{
			x += dx;
			pRCP->charPos = cp;
		}
	}

	return false;
}

void CHTMLView::DrawLines(CDC* pDC)
{
	TOKENLIST& tiList = m_tokens;
	POSITION pos = tiList.GetHeadPosition();
	CTokenInfo ti;
	CFont *pOldFont, font;
	CRect rect;
	
	GetClientRect(rect);
	rect.left += m_rcMargins.left;

	int x = rect.left;
	int y = rect.top + m_rcMargins.top;

	POSITION linePos = m_page.FindIndex(GetScrollPos(SB_VERT));
	POSITION tiPos;
	TEXTLINE line;
	
	pOldFont = pDC->GetCurrentFont();

	while (linePos && y <= rect.bottom) 
	{
		pDC->MoveTo(x, y);

		line = m_page.GetNext(linePos);

		tiPos = line.first.pTokenInfo;
		ASSERT(tiPos);
		
		// Get first token
		ti = tiList.GetNext(tiPos);
		
		PrepareFont(&font, ti.m_nStyle);
		pDC->SelectObject(&font);

		if (ti.m_nStyle & TI_HYPERLINK)
			pDC->SetTextColor(RGB(0,0,255));
		else
			pDC->SetTextColor(RGB(0,0,0));

		// Set text pointer and text length
		ti.m_pText += line.first.nIndex;
		
		if (line.first.pTokenInfo == line.last.pTokenInfo)
			ti.m_nLen = line.last.nIndex - line.first.nIndex + 1;
		else
			ti.m_nLen -= line.first.nIndex;

		pDC->ExtTextOut(x, y, 0, NULL, ti.m_pText, ti.m_nLen, NULL);

		x += CalcTotalWidth(ti.m_pWidths + line.first.nIndex, ti.m_nLen);
		pDC->MoveTo(x, y);

		pDC->SelectObject(pOldFont);
		font.DeleteObject();

		// End first token
		
		if (line.first.pTokenInfo != line.last.pTokenInfo)
		{
			// Get intermediate tokens
			while (tiPos != line.last.pTokenInfo)
			{
				ASSERT(tiPos);
				ti = tiList.GetNext(tiPos);

				PrepareFont(&font, ti.m_nStyle);
				pDC->SelectObject(&font);

				if (ti.m_nStyle & TI_HYPERLINK)
					pDC->SetTextColor(RGB(0,0,255));
				else
					pDC->SetTextColor(RGB(0,0,0));
				
				pDC->ExtTextOut(x, y, 0, NULL, ti.m_pText, ti.m_nLen, NULL);

				x += CalcTotalWidth(ti.m_pWidths, ti.m_nLen);
				pDC->MoveTo(x, y);

				pDC->SelectObject(pOldFont);
				font.DeleteObject();
			} 
			// End intermediate tokens

			// Get last token
			ti = tiList.GetAt(tiPos);

			PrepareFont(&font, ti.m_nStyle);
			pDC->SelectObject(&font);

			if (ti.m_nStyle & TI_HYPERLINK)
				pDC->SetTextColor(RGB(0,0,255));
			else
				pDC->SetTextColor(RGB(0,0,0));
			
			pDC->ExtTextOut(x, y, 0, NULL, ti.m_pText, line.last.nIndex + 1, NULL);

			x += CalcTotalWidth(ti.m_pWidths, line.last.nIndex + 1);
			pDC->MoveTo(x, y);

			pDC->SelectObject(pOldFont);
			font.DeleteObject();
			// End last token
		}

		y += line.TotalHeight();
		x = rect.left;
	}
}

void CHTMLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	if (!m_page.IsEmpty()) {
		BreakLines();
		SetScrollPage();
	}	
}

CTokenInfo* CHTMLView::GetTokenInfoAtPt(const CPoint& pt)
{
	if (m_page.IsEmpty()) return NULL;

	RELCHARPOS rcp;

	if(GetCharFromPt(pt, &rcp))
		return &m_tokens.GetAt(rcp.charPos.pTokenInfo);
	else
		return NULL;
}

void CHTMLView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	RELCHARPOS rcp;

	if(GetCharFromPt(point, &rcp, &m_ptSelStart))
	{
		if (m_bTextSelected)
			InvalidateRect(m_rcSelection);

		m_bTextSelected = false;
		m_bTextSelecting = true;
		m_selStart = rcp.charPos;
		m_selEnd = rcp.charPos;
	}
}

void CHTMLView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (m_bTextSelecting)
	{
		::SetCursor(::LoadCursor(NULL, IDC_IBEAM));	// OS' cursor

		RELCHARPOS rcp;
		CPoint ptEdge;
		
		if(GetCharFromPt(point, &rcp, &ptEdge))
		{
			InvalidateRect(m_rcSelection);

			m_bTextSelected = true;
			m_ptSelEnd = ptEdge;
			m_selEnd = rcp.charPos; // Selection includes the last char.

			/*if (point.x <= m_ptSelEnd.x && rcp.charPos != m_selStart) {
				GetPrevWidth(&m_selEnd); // Move it to the previous character.
				m_bTextSelected = true;
			}	
			else if (rcp.charPos == m_selStart)
				m_bTextSelected = false;
			else
				m_bTextSelected = true;*/

			DrawSelection(NULL);
			InvalidateRect(m_rcSelection);
		}
	}
	else if (IsTokenStyleAt(point, TI_HYPERLINK))
		::SetCursor(AfxGetApp()->LoadCursor(IDC_HAND)); // My own cursor
	else
		::SetCursor(::LoadCursor(NULL, IDC_ARROW));	// OS' cursor
}

void CHTMLView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	RELCHARPOS rcp;

	m_bTextSelecting = false;	// Always turn off the selection, just in case.

	if(!GetCharFromPt(point, &rcp))
		return;

	CTokenInfo& ti = m_tokens.GetAt(rcp.charPos.pTokenInfo);

	if (ti.m_nStyle & TI_HYPERLINK)
		OnLink(ti.m_szData);

	/*CTokenInfo* p = GetTokenInfoAtPt(point);
	if (p && p->m_nStyle & TI_HYPERLINK)
		OnLink(p->m_szData);*/
}

void CHTMLView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int nPixelInc = 0, nPosInc = 0;
	int nCurPos = GetScrollPos(SB_VERT);
	int nMax = GetScrollLimit(SB_VERT);
	CRect clipRect;
	GetClientRect(clipRect);
	clipRect.top = m_rcMargins.top;
	
	switch (nSBCode) {
		case SB_LINEDOWN: 
			if (nCurPos < nMax) {
				nPosInc = 1;
				nPixelInc = -GetLineTotalHeight(nCurPos);
			}
			break;
		case SB_LINEUP:
			if (nCurPos > 0) {
				nPosInc = -1;
				nPixelInc = GetLineTotalHeight(nCurPos - 1);
			}
			break;
		case SB_PAGEDOWN: {
			int i = nCurPos, y = 0;

			while (i < nMax) {
				y += GetLineTotalHeight(i);
				if (y + clipRect.top <= clipRect.bottom) {
					i++;
					nPixelInc = -y;
				}
				else break;
			}
			nPosInc = i - nCurPos;
			break;
		}
		case SB_PAGEUP: {
			int i = nCurPos;

			while (i > 0 && clipRect.bottom - nPixelInc > clipRect.top)
				nPixelInc += GetLineTotalHeight(--i);

			nPosInc = i - nCurPos;
			break;
		}
		case SB_THUMBTRACK:
			if (nCurPos < (int)nPos) {
				while (nCurPos + nPosInc < (int)nPos && nCurPos + nPosInc < nMax) 
				{
					nPixelInc -= GetLineTotalHeight(nCurPos + nPosInc);
					nPosInc++;
				}
			}
			else if (nCurPos > (int)nPos){
				nPosInc--;
				while (nCurPos + nPosInc >= (int)nPos && nCurPos + nPosInc >= 0) {
					nPixelInc += GetLineTotalHeight(nCurPos + nPosInc);
					nPosInc--;
				}
				nPosInc++;
			}
			break;
		default:
			nPosInc = 0;
	}

	if (nPosInc)
	{	
		ScrollWindow(0, nPixelInc, NULL, clipRect);
		SetScrollPos(SB_VERT, nCurPos + nPosInc);
		
		UpdateWindow();
	}
	
	CView::OnVScroll(nSBCode, nPos, pScrollBar);
}

int CHTMLView::CalcScrollPage() 
{
	CRect clientRect;
	GetClientRect(clientRect);

	int nTotal = 0;
	int nCount = m_page.GetCount();
	for (int i = 0; i < nCount; i++)
		nTotal += GetLineTotalHeight(i);

	return (nTotal) ? clientRect.bottom / (nTotal / nCount):0;
}

int CHTMLView::GetScrollPage()
{
	SCROLLINFO si;
	GetScrollInfo(SB_VERT, &si, SIF_PAGE);
	return si.nPage;
}

void CHTMLView::SetScrollPage()
{
	SCROLLINFO si;

	si.cbSize = sizeof(SCROLLINFO); 
    si.fMask = SIF_PAGE | SIF_RANGE; 
	si.nPage = CalcScrollPage();
	si.nMax = m_page.GetCount();
	//si.nMax += si.nPage - 1;
	si.nMin = 0;
	SetScrollInfo(SB_VERT, &si);
}

int CHTMLView::GetLineTotalHeight(int nLine)
{
	return m_page.GetAt(m_page.FindIndex(nLine)).TotalHeight();
}

BOOL CHTMLView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// The window class should be modified to set
	// hCursor whit NULL value. Its needed to change
	// mouse curso to hand cursor when its over a hyperlink.
	// In that way it won't be necesary to capture cursor.
	
	static const char szClassName[] = "HTMLViewClass";

	if (!s_bIsClassRegistered)
	{
		WNDCLASS wc;

		// Start setting with common view values
		wc.style = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
		wc.lpfnWndProc = AfxWndProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = 0;
		wc.hInstance = AfxGetInstanceHandle();
		wc.hCursor = NULL;
		wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
		wc.lpszMenuName = NULL;
		wc.lpszClassName = szClassName;

		// hIcon needs a special code that I borrowed from MFC code
		HINSTANCE hInst = AfxFindResourceHandle(
			MAKEINTRESOURCE(AFX_IDI_STD_FRAME), RT_GROUP_ICON);
		if ((wc.hIcon = ::LoadIcon(hInst, MAKEINTRESOURCE(AFX_IDI_STD_FRAME))) == NULL)
		{
			// use default icon
			wc.hIcon = ::LoadIcon(NULL, IDI_APPLICATION);
		}

		// Register window class
		if (!AfxRegisterClass(&wc)) {
			TRACE0("Failed to register view window class\n");
			return 0;
		}
		
		s_bIsClassRegistered = true;
	}

	// Say to the MFC to use this wnd class
	cs.lpszClass = szClassName;
	
	return CView::PreCreateWindow(cs);
}

void CHTMLView::OnHighlightText() 
{
	HIGHLIGHT hl;

	hl.start = m_selStart;
	hl.end = m_selEnd;

	// Let params in order
	if (!IsSelectionInOrder())
	{
		CHARPOS aux = hl.start;
		hl.start = hl.end;
		hl.end = aux;
	}

	hl.nFirstChar = GetAbsCharPos(hl.start);
	hl.nLastChar = GetAbsCharPos(hl.end);

	m_hlList.AddTail(hl);
	InvalidateRect(m_rcSelection);
	SetModifiedFlag();
}

void CHTMLView::OnUpdateHighlightText(CCmdUI* pCmdUI) 
{
	if (m_bTextSelected)
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

void CHTMLView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	CView::OnPrepareDC(pDC, pInfo);
}
