// LibraryDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ITBReader.h"
#include "LibraryDlg.h"
#include "CommonTypes.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CString	g_strCurrentLibrary;
extern bool		g_bIsConnected;
extern UINT InternetConnectionThreadProc(LPVOID pParam);
extern void InternetDisconnect();

#define WM_CONNECTED WM_USER + 5

extern CONTLINELIST	g_currContentsList;

CStringList CLibraryDlg::s_libraryList;

/////////////////////////////////////////////////////////////////////////////
// CLibraryDlg dialog
CLibraryDlg::CLibraryDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CLibraryDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLibraryDlg)
	m_bAutoClose = TRUE;
	//}}AFX_DATA_INIT
}


void CLibraryDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLibraryDlg)
	DDX_Control(pDX, IDC_PASSWORD, m_editPassword);
	DDX_Control(pDX, IDC_USERNAME, m_editUserName);
	DDX_Control(pDX, IDC_STATUS, m_editStatus);
	DDX_Control(pDX, IDC_COMBOURL, m_comboURL);
	DDX_Check(pDX, IDC_AUTO_CLOSE, m_bAutoClose);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CLibraryDlg, CDialog)
	//{{AFX_MSG_MAP(CLibraryDlg)
	ON_BN_CLICKED(IDC_CONNECT, OnConnect)
	ON_BN_CLICKED(IDC_DISCONNECT, OnDisconnect)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLibraryDlg message handlers

BOOL CLibraryDlg::Create() 
{
	return CDialog::Create(CLibraryDlg::IDD);
}

void CLibraryDlg::OnConnect() 
{
	m_comboURL.GetWindowText(g_strCurrentLibrary);

	if (m_comboURL.FindStringExact(-1, g_strCurrentLibrary) == CB_ERR)
	{
		m_comboURL.AddString(g_strCurrentLibrary);
		s_libraryList.AddTail(g_strCurrentLibrary);
	}

	m_editUserName.GetWindowText(g_strUserName);
	m_editPassword.GetWindowText(g_strUserPassword);

	if (!MakeValidCGIText(g_strUserName)) {
		WarnUserOfInvalidCGI(_T("User Name"));
		return;
	}
	if (!MakeValidCGIText(g_strUserPassword)) {
		WarnUserOfInvalidCGI(_T("Password"));
		return;
	}

	CWaitCursor wc;
	
	InternetConnectionThreadProc(GetParent()->GetSafeHwnd());

	if (!g_bIsConnected) {
		m_editStatus.SetWindowText(TEXT("ERROR: UNABLE TO CONNECT"));
		return;
	}
	
	LRESULT res = GetParent()->SendMessage(WM_CONNECTED);
		
	if (res == CONNECT_ERROR_INVALIDUSER)
		m_editStatus.SetWindowText(TEXT("ERROR: INVALID USER OR PASSWORD"));
	else if (res == CONNECT_ERROR_REQUEST)
		m_editStatus.SetWindowText(TEXT("ERROR: UNABLE TO CHECK USER"));
	else if (res == CONNECT_OK) 
	{
		g_currContentsList.RemoveAll();	// Current Contents list
		m_editStatus.SetWindowText(TEXT("CONNECTED"));
		if (m_bAutoClose) 
			OnCancel();
	}
	else
		m_editStatus.SetWindowText(TEXT("ERROR: UNKNOWN STATE"));
}

BOOL CLibraryDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	if (s_libraryList.IsEmpty())
		s_libraryList.AddTail(_T("localhost"));

	POSITION pos = s_libraryList.GetHeadPosition();

	while (pos) 
		m_comboURL.AddString(s_libraryList.GetNext(pos));

	m_comboURL.SetWindowText(g_strCurrentLibrary);
	m_editUserName.SetWindowText(g_strUserName);
	m_editStatus.SetWindowText((g_bIsConnected) ? 
		_T("CONNECTED"):_T("DISCONNECTED"));

	//Debug, temporal
#ifdef _DEBUG
	m_editPassword.SetWindowText("guest");
	m_editUserName.SetWindowText("guest");
#endif _DEBUG
	// End Debug
	
	return TRUE;
}

void CLibraryDlg::OnCancel() 
{
	CDialog::OnCancel();
}

void CLibraryDlg::OnDisconnect() 
{
	InternetDisconnect();
	if (GetSafeHwnd())
		m_editStatus.SetWindowText(TEXT("NOT CONNECTED"));
}

bool CLibraryDlg::MakeValidCGIText(CString& str)
{
	int nLen = str.GetLength();

	for (int i = 0; i < nLen; i++)
	{
		if (str[i] == '?' || str[i] == '&' || str[i] == '+') 
			return false;
		else if (str[i] == ' ') 
			str.SetAt(i, '+');	// CGI convention
	}
	return true;
}

void CLibraryDlg::WarnUserOfInvalidCGI(LPCTSTR field)
{
	CString str;
	AfxFormatString1(str, IDS_INVALID_CGI_STRING, field);
	//ShowWindow(SW_HIDE);
	MessageBox(str, _T("Invalid Entry"), MB_ICONERROR | MB_OK );
	//ShowWindow(SW_SHOW);
}
