// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "ITBReader.h"

#include "LibraryConnection.h"
#include "MainFrm.h"
#include "PictureView.h"
#include "NoteView.h"
#include "LibraryDlg.h"

#include "MainSheet.h"
#include "PrintDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define NUMBEROFIMAGES			10	 // Amount of buttons
#define COMBOBOXNEXTBUTTON		16	 // CB's next button
#define COMBOBOXWIDTH			78	 // Just to hold the maximum text length
#define COMBOBOXTEXTLIMIT		9	 // Page XXXX => It has 9 digits
#define MAXPAGE					9999 // Maximum page number
#define PAGECACHE				10   // Number of visited page numbers stored in the combo box

#define WM_CONNECT				WM_USER + 5
#define WM_POST_CREATE_CLIENT	WM_USER + 6

#define COMBOBOX_INDEX 14
#define COMBOBOX_WIDTH 150
#define COMBOBOX_HEIGHT 150

extern bool	g_bIsConnected;

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_LIBRARY_CONNECTION, OnLibraryConnection)
	ON_COMMAND(ID_NEXT_PAGE, OnNextPage)
	ON_COMMAND(ID_PREV_PAGE, OnPrevPage)
	ON_COMMAND(ID_MAIN_SHEET, OnMainSheet)
	ON_COMMAND(ID_FILE_PRINT, OnPrint)
	ON_COMMAND(ID_EDIT_FIND, OnEditFind)
	ON_COMMAND(ID_VIEW_CONTENTS, OnViewContents)
	ON_COMMAND(ID_VIEW_INDEX, OnViewIndex)
	ON_COMMAND(ID_FILE_PRINT_CONTENTS_SETUP, OnFilePrintContentsSetup)
	ON_COMMAND(ID_VIEW_BOOK_INFO, OnViewBookInfo)
	ON_COMMAND(ID_FILE_SAVE_EX, OnFileSaveEx)
	ON_COMMAND(ID_FILE_HOMEPAGE, OnFileHomepage)
	ON_COMMAND(ID_TOGGLE_NOTE, OnToggleNote)
	ON_UPDATE_COMMAND_UI(ID_TOGGLE_NOTE, OnUpdateToggleNote)
	ON_COMMAND(ID_VIEW_AUTHOR_INFO, OnViewAuthorInfo)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_CONNECT, OnConnect)
	ON_MESSAGE(WM_GOTO_PAGE, OnGoToPage)
	ON_MESSAGE(WM_NOTE_CLICK, OnNoteClick)
	ON_MESSAGE(WM_ADD_NOTE, OnAddNote)
	ON_MESSAGE(WM_DELETE_NOTE, OnDeleteNote)
	ON_MESSAGE(WM_PUSH_PICTURE, OnPushPicture)
	ON_MESSAGE(WM_POST_CREATE_CLIENT, OnPostCreateClient)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_hPageCBEdit = NULL;
	
	m_nNoteViewHeight = 50;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	// Create page number combobox
	CRect rect(-COMBOBOX_WIDTH, -COMBOBOX_HEIGHT, 0, 0);
	if (!m_pageCB.Create(WS_VISIBLE | WS_CHILD | CBS_DROPDOWN |
		CBS_AUTOHSCROLL | WS_VSCROLL | CBS_HASSTRINGS, rect, &m_wndToolBar,
		ID_BOOKMARK_LIST))
	{
		TRACE0("Failed to create page number combobox\n");
		return -1;      // fail to create
	}

	// Set Combobox font
	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	if (hFont == NULL)
		hFont = (HFONT)GetStockObject(ANSI_VAR_FONT);

  	m_pageCB.SendMessage(WM_SETFONT, (WPARAM)hFont);

	// Load the toolbar
	if (!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to load toolbar\n");
		return -1;      // fail to create
	}

	// Attach the page number combobox to the toolbar
	m_wndToolBar.SetButtonInfo(COMBOBOX_INDEX, ID_BOOKMARK_LIST, TBBS_SEPARATOR, COMBOBOX_WIDTH);
	m_wndToolBar.GetItemRect(COMBOBOX_INDEX, rect);
	m_pageCB.SetWindowPos(NULL, rect.left, rect.top, 0, 0, 
		SWP_NOZORDER | SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOCOPYBITS);
	m_pageCB.ShowWindow(SW_SHOW);

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	return 0;
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT /*lpcs*/,
	CCreateContext* pContext)
{
	// Create main splitter
	if (!m_wndMainSplitter.CreateStatic(this,	1, 2)) {
		TRACE0("Error: Creating main splitter");
		return FALSE;
	}

	// Neste second splitter
	if (!m_wndNestedSplitter.CreateStatic(&m_wndMainSplitter, 2, 1, 
		WS_CHILD | WS_VISIBLE, m_wndMainSplitter.IdFromRowCol(0, 0))) 
	{
		TRACE0("Error: Creating nested splitter");
		return FALSE;
	}

	// Attach first pane
	if (!m_wndNestedSplitter.CreateView(0, 0, pContext->m_pNewViewClass,
		CSize(500, 200), pContext)) 
	{
		TRACE0("Error: Attaching text pane");
		return FALSE;
	}

	// Attach second pane
	if (!m_wndNestedSplitter.CreateView(1, 0, RUNTIME_CLASS(CNoteView),
		CSize(300, 100), pContext)) 
	{
		TRACE0("Error: Attaching note pane");
		return FALSE;
	}

	// Attach third pane
	if (!m_wndMainSplitter.CreateView(0, 1, RUNTIME_CLASS(CPictureView),
		CSize(50, 100), pContext)) 
	{
		TRACE0("Error: Attaching picture pane");
		return FALSE;
	}

	PostMessage(WM_POST_CREATE_CLIENT);

	return TRUE;
}

LRESULT CMainFrame::OnPostCreateClient(WPARAM wParam, LPARAM lParam)
{
	CRect rect;
	GetClientRect(rect);

	m_wndMainSplitter.SetColumnInfo(0, int(rect.Width() * 0.85), 0);
	m_wndNestedSplitter.SetRowInfo(0, int(rect.Height() * 0.85), 0);
	m_wndMainSplitter.RecalcLayout();
	m_wndNestedSplitter.RecalcLayout();

	return 0L;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers


BOOL CMainFrame::DestroyWindow() 
{
#ifdef _DEBUG
	if (g_RICount != 0) {
		CString str;
		str.Format(_T("RequestInfoStruct leak = %d"), g_RICount);
		MessageBox(str);
	}
#endif
	
	//m_pageCB.Detach();	// It's destroyed by the command bar
	
	return CFrameWnd::DestroyWindow();
}

BOOL CMainFrame::PreTranslateMessage(MSG* pMsg) 
{
	/*
	if (pMsg->hwnd == m_hPageCBEdit &&
		pMsg->message == WM_KEYDOWN &&
		pMsg->wParam == VK_RETURN) 
	{
		TCHAR szString[COMBOBOXTEXTLIMIT + 1];
		if (m_pageCB.GetWindowText(szString, COMBOBOXTEXTLIMIT + 1) == 0)
			return TRUE;

		int nPage = ParseComboBoxText(szString);

		if (nPage < 0) {
			AfxMessageBox(TEXT("Invaild Entry"));
			return TRUE;
		}
		
		// There is a problem if the combobox is showing
		// its list, so I hide the list just in case.
		m_pageCB.ShowDropDown(FALSE);
		
		// Limit the number of strings stored
		m_pageCB.SetCurSel(0);
		int nCount = m_pageCB.GetCount();	
		if (nCount > PAGECACHE)
			m_pageCB.DeleteString(nCount - 1);

		// Add the string to the ComboBox
		long dwIndex = m_pageCB.InsertString(0, szString);
		
		// Set associated data item
		if (dwIndex != CB_ERR && dwIndex != CB_ERRSPACE) 
			m_pageCB.SetItemData(dwIndex, nPage);

		// Let's go to the selected page
		PostMessage(WM_GOTO_PAGE, (WPARAM) nPage);
		
		return TRUE;
	}
	*/

	return CFrameWnd::PreTranslateMessage(pMsg);
}

int CMainFrame::ParseComboBoxText(TCHAR* szString)
{
	// Look for the first digit and don't care about 
	// its prefix or subfix.
	TCHAR* p;
	int nPage;

	for (p  = szString; !_istdigit(*p) && *p != '\0'; p++);
	nPage = (*p == '\0') ? -1:_ttoi(p);
	if (nPage >= 0 && nPage <= MAXPAGE) {
		_tcscpy(szString, _T("Page "));
		_itot(nPage, szString + 5, 10);
	}
	else
		nPage = -1;

	return nPage;
}

void CMainFrame::UpdatePageNumberText()
{
	ASSERT(GetCurrentPage() == LIBRARY_PAGE ||
		(GetCurrentPage() >= 0 && GetCurrentPage() <= MAXPAGE));
	
	if (GetCurrentPage() == LIBRARY_PAGE) {
		m_pageCB.SetWindowText(_T("HomePage"));
	}
	else 
	{
		TCHAR szString[COMBOBOXTEXTLIMIT + 1];
		memcpy(szString, _T("Page "), sizeof(TCHAR) * 5);

		_itot(GetCurrentPage(), szString + 5, 10);

		m_pageCB.SetWindowText(szString);
	}
	
	SetFocus();
}

void CMainFrame::OnSelChangePageCombobox()
{
	long dwIndex = m_pageCB.GetCurSel();
	
	if (dwIndex != CB_ERR)
		OnGoToPage(m_pageCB.GetItemData(dwIndex));
}

void CMainFrame::OnLibraryConnection() 
{
	CLibraryDlg dlg;

	dlg.DoModal();
}

LRESULT CMainFrame::OnConnect(WPARAM wParam, LPARAM lParam)
{
	REQUESTINFO ri;
	
	ri.strObjectName.Format(
		_T("/ITBLibrary.dll?Connect?UserName=%s&Password=%s"), 
		GetITBUsrName(), GetITBUsrPsw());
	
	ri.lpszVerb = NULL;	// GET
	ri.nReadFuncType = RFT_VOID;
	ri.hwndRequester = NULL;
	ri.nPage = PAGE_INDEPENDENT;

	UINT uRet = InternetRequestThreadProc(&ri);

	if (uRet == IR_ANSW_404)
		return CONNECT_ERROR_INVALIDUSER;
	else if (uRet != IR_OK)
		return CONNECT_ERROR_REQUEST;

	OnGoToPage(LIBRARY_PAGE);

	return CONNECT_OK;
}

LRESULT CMainFrame::OnGoToPage(WPARAM wParam, LPARAM lParam)
{
	if (!g_bIsConnected) {
		AfxMessageBox(IDS_ERROR_DISCONNECTED);
		return FALSE;
	}

	int nPage = (int)wParam;

	if(nPage != LIBRARY_PAGE &&	(nPage < 0 || nPage > MAXPAGE))
		return FALSE;

	CWaitCursor wc;

	::SetCurrentPage((int)wParam);
	if (lParam != 0)
		::SetCurrentBook((UINT)lParam);

	UpdatePageNumberText();
	
	if (wParam == LIBRARY_PAGE) {
		::SetCurrentPage(0);
		::SetCurrentBook(0);
		GetActiveDocument()->UpdateAllViews(NULL, 
			::GetCurrentPage(), (CObject*)1);
	}
	else
		GetActiveDocument()->UpdateAllViews(NULL, 
			::GetCurrentPage(), (CObject*)::GetCurrentBook());

	return TRUE;
}

void CMainFrame::OnNextPage()
{
	OnGoToPage(GetCurrentPage() + 1);
}

void CMainFrame::OnPrevPage()
{
	OnGoToPage(GetCurrentPage() - 1);
}

void CMainFrame::OnMainSheet() 
{
	CMainSheet	dlg(TEXT(""), this, 0);

	dlg.DoModal();
}

void CMainFrame::OnViewContents() 
{
	CMainSheet	dlg(TEXT(""), this, 0);

	dlg.DoModal();
}

void CMainFrame::OnEditFind() 
{
	CMainSheet	dlg(TEXT(""), this, 1);

	dlg.DoModal();
}

void CMainFrame::OnViewIndex() 
{
	CMainSheet	dlg(TEXT(""), this, 2);

	dlg.DoModal();	
}

void CMainFrame::OnViewAuthorInfo() 
{
	CMainSheet	dlg(TEXT(""), this, 3);

	dlg.DoModal();	
}

void CMainFrame::OnViewBookInfo() 
{
	CMainSheet	dlg(TEXT(""), this, 4);

	dlg.DoModal();
}

void CMainFrame::ShowNoteView(bool bShow /*= true*/)
{
	int cyHTML, cyHTMLmin, cyNote, cyNoteMin;
	CString str;

	m_wndNestedSplitter.GetRowInfo(0, cyHTML, cyHTMLmin);
	m_wndNestedSplitter.GetRowInfo(1, cyNote, cyNoteMin);

	if (bShow && cyNote == 0)
	{
		m_wndNestedSplitter.SetRowInfo(0, cyHTML - m_nNoteViewHeight, 
			cyHTMLmin);
	    m_wndNestedSplitter.RecalcLayout();
	}
	else if (!bShow && cyNote != 0)
	{
		m_nNoteViewHeight = cyNote;
		m_wndNestedSplitter.SetRowInfo(0, cyHTML + cyNote, cyHTMLmin);
	    m_wndNestedSplitter.RecalcLayout();
	}
}

LRESULT CMainFrame::OnNoteClick(WPARAM wParam, LPARAM lParam /*Pic Index*/)
{
	ShowNoteView();
	
	NOTEKEY nk;
	nk.nBook = GetCurrentBook();
	nk.nPage = GetCurrentPage();
	nk.nIndex = (int) lParam;

	m_wndNestedSplitter.GetPane(1, 0)->SendMessage(WM_NOTE_CLICK, 
		(WPARAM)&nk, lParam);

	return TRUE;
}

LRESULT CMainFrame::OnAddNote(WPARAM wParam, LPARAM lParam /*Pic Index*/)
{
	ShowNoteView();

	NOTEKEY nk;
	nk.nBook = GetCurrentBook();
	nk.nPage = GetCurrentPage();
	nk.nIndex = (int) lParam;

	m_wndNestedSplitter.GetPane(1, 0)->SendMessage(WM_ADD_NOTE, 
		(WPARAM)&nk, lParam);
	
	return TRUE;
}

LRESULT CMainFrame::OnDeleteNote(WPARAM wParam, LPARAM lParam /*Pic Index*/)
{
	ShowNoteView(false);

	NOTEKEY nk;
	nk.nBook = GetCurrentBook();
	nk.nPage = GetCurrentPage();
	nk.nIndex = (int) lParam;

	return m_wndNestedSplitter.GetPane(1, 0)->SendMessage(WM_DELETE_NOTE, 
		(WPARAM)&nk, lParam);
}

void CMainFrame::OnPrint() 
{
	/*PRINTDLG pd;

	pd.cbStruct = sizeof(pd);
	pd.hwndOwner = AfxGetMainWnd()->GetSafeHwnd();
	//pd.hdc;
	pd.dwFlags = 0;//PD_SELECTALLPAGES ;
	//pd.rcMinMargin;
	//pd.rcMargin;
	//pd.hinst;
	pd.lCustData = 0;
	//pd.pfnPrintHook;
	//pd.pszPrintTemplateName;
	//pd.hglbPrintTemplateResource;

	PrintDlg(&pd);	*/
}

void CMainFrame::OnFilePrintContentsSetup() 
{
	CPrintDlg dlg;
	
	dlg.DoModal();
}

void CMainFrame::OnFileSaveEx() 
{
	m_wndNestedSplitter.GetPane(0, 0)->SendMessage(WM_SAVE_CONTENTS);
	m_wndNestedSplitter.GetPane(1, 0)->SendMessage(WM_SAVE_CONTENTS);
}

void CMainFrame::OnFileHomepage() 
{
	OnGoToPage(LIBRARY_PAGE);
}

void CMainFrame::OnToggleNote() 
{
	int cx,cxmin;
	m_wndNestedSplitter.GetRowInfo(1, cx, cxmin);
	ShowNoteView((cx) ? false:true);
}

void CMainFrame::OnUpdateToggleNote(CCmdUI* pCmdUI) 
{
	int cx,cxmin;
	m_wndNestedSplitter.GetRowInfo(1, cx, cxmin);
	pCmdUI->SetCheck((cx) ? TRUE:FALSE);	
}

void CMainFrame::OnClose() 
{
	if (!m_wndNestedSplitter.GetPane(0, 0)->SendMessage(WM_SAVE_CONTENTS) &&
		!m_wndNestedSplitter.GetPane(1, 0)->SendMessage(WM_SAVE_CONTENTS) && 
		AfxMessageBox(IDS_ERROR_SAVE_NOTE, MB_ICONERROR | MB_YESNO | MB_DEFBUTTON2) == IDNO)
		return;

	CFrameWnd::OnClose();
}

LRESULT CMainFrame::OnPushPicture(WPARAM wParam, LPARAM lParam)
{
	return m_wndMainSplitter.GetPane(0, 1)->SendMessage(WM_PUSH_PICTURE, wParam, lParam);
}
