// PictureView.cpp : implementation file
//

#include "stdafx.h"
#include "ITBReader.h"
#include "PictureView.h"
#include "LibraryConnection.h"
#include "DIBWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define	INTERPICFACTOR	0.05

CRITICAL_SECTION g_CriticalSection;

/////////////////////////////////////////////////////////////////////////////
// CPictureView

IMPLEMENT_DYNCREATE(CPictureView, CScrollView)

CPictureView::CPictureView()
{
	m_nLeftMargin = 5;
	m_nRightMargin = 5;
	m_nInterPicGap = 4;
	
	m_pActiveNotePicWnd = NULL;

	::InitializeCriticalSection(&g_CriticalSection);
}

CPictureView::~CPictureView()
{
	Empty();
	::DeleteCriticalSection(&g_CriticalSection);
}

BEGIN_MESSAGE_MAP(CPictureView, CScrollView)
	//{{AFX_MSG_MAP(CPictureView)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_REQUEST_COMPLETED, OnRequestCompleted)
	ON_MESSAGE(WM_NOTE_CLICK, OnNoteClick)
	ON_MESSAGE(WM_ADD_NOTE, OnAddNote)
	ON_MESSAGE(WM_DELETE_NOTE, OnDeleteNote)
	ON_MESSAGE(WM_PUSH_PICTURE, OnPushPicture)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPictureView drawing

void CPictureView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	SetScrollSizes(MM_TEXT, CSize(0, 0));
}

void CPictureView::OnDraw(CDC* pDC)
{
}

/////////////////////////////////////////////////////////////////////////////
// CPictureView diagnostics

#ifdef _DEBUG
void CPictureView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CPictureView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPictureView message handlers

BOOL CPictureView::PreCreateWindow(CREATESTRUCT& cs) 
{
	cs.style |= WS_CLIPCHILDREN;	// To avoid fliking
	
	return CScrollView::PreCreateWindow(cs);
}

void CPictureView::OnSize(UINT nType, int cx, int cy) 
{
	POSITION pos = m_picList.GetHeadPosition();
	
	if (cx >= 0 && pos)
	{
		CPictureWnd* pPicWnd;
		CRect rect;
		CSize totalSize(0, 0);
		int nTop;

		pPicWnd = m_picList.GetNext(pos);
		
		pPicWnd->GetWindowRect(rect);
		ScreenToClient(rect);

		nTop = rect.top;
		rect.left = m_nLeftMargin;
		rect.right = cx - m_nRightMargin;

		// Recalc layout
		//m_nInterPicGap = (int)(cx * INTERPICFACTOR);
		pPicWnd->RecalcShadow(rect.Width()); // Call this function just once

		pPicWnd->ScaleMove(&rect);
		totalSize.cy = m_nInterPicGap + rect.bottom - rect.top;
		rect.top = rect.bottom + m_nInterPicGap;
		
		while (pos) {
			pPicWnd = m_picList.GetNext(pos);
			pPicWnd->ScaleMove(&rect);
			totalSize.cy += m_nInterPicGap + rect.bottom - rect.top;
			rect.top = rect.bottom + m_nInterPicGap;
		}

		// Following code handles, in an ugly way, an extrange case 
		// that happen in some situations.
		CRect clientRect;
		GetClientRect(clientRect);

		if (rect.bottom < clientRect.bottom && nTop < m_nInterPicGap)
			OffsetPictures(clientRect.bottom - rect.bottom);
		// End of the extrange case

		SetScrollSizes(MM_TEXT, totalSize);
	}
	
	CScrollView::OnSize(nType, cx, cy);
}

void CPictureView::OffsetPictures(int cy)
{
	POSITION pos = m_picList.GetHeadPosition();
	CWnd* pWnd;
	CRect rect;
	while (pos)
	{
		pWnd = m_picList.GetNext(pos);
		pWnd->GetWindowRect(rect);
		ScreenToClient(rect);
		rect.OffsetRect(0, cy);
		pWnd->MoveWindow(rect, FALSE);
	}
}

void CPictureView::Empty() 
{
	EnterCriticalSection(&g_CriticalSection);
	
	m_pActiveNotePicWnd = NULL;

	CPictureWnd* p;
	while (!m_picList.IsEmpty()) 
	{
		p = m_picList.RemoveTail();
		p->DestroyWindow();
		delete p;
	}
	
	SetScrollSizes(MM_TEXT, CSize(0,0));
	
	LeaveCriticalSection(&g_CriticalSection);
}

void CPictureView::OnUpdate(CView* pSender, LPARAM lPageNumber, CObject* pHint) 
{
	if (pHint == NULL) return;	// NULL => MFC default update
	
	Empty();
	
	REQUESTINFO* pRequestInfo = new REQUESTINFO;
	if (!pRequestInfo) return;

	pRequestInfo->strObjectName.Format(
		_T("/ITBLibrary.dll?GetPictureHeaders?UserName=%s&Password=%s&Book=%d&Page=%d"), 
		GetITBUsrName(), GetITBUsrPsw(), ::GetCurrentBook(), lPageNumber);

	pRequestInfo->lpszVerb = NULL;
	pRequestInfo->nReadFuncType = RFT_ASCII;
	pRequestInfo->hwndRequester = GetSafeHwnd();
	pRequestInfo->nPage = (int)lPageNumber;
	pRequestInfo->uBook = ::GetCurrentBook();

	AfxBeginThread(InternetRequestThreadProc, pRequestInfo);
}

int CPictureView::AddPicture(const PICTUREINFO& pi, CRect& rect)
{
	CPictureWnd* pPicWnd;
	int nHeight;
	
	switch (pi.nType)
	{
		case OT_DIB:
			pPicWnd = new CDIBWnd();
			pPicWnd->Create(pi, rect, this, 300 + m_picList.GetCount());
			nHeight = rect.Height();
			rect.top = rect.bottom + m_nInterPicGap;
			break;
		default:
			MessageBox(TEXT("default add picture"));
			pPicWnd = NULL;
			break;
	}
	
	if (pPicWnd) m_picList.AddTail(pPicWnd);
	
	return nHeight;
}

void CPictureView::AddErrorPicture(const PICTUREINFO& pi, CRect& rect)
{
	MessageBox(TEXT("AddErrorPicture"));
}

LRESULT CPictureView::OnRequestCompleted(WPARAM wParam, LPARAM lParam)
{
	REQUESTINFO* pRequestInfo = (REQUESTINFO*)lParam;
	
	if (wParam == FALSE) 
	{
		delete pRequestInfo;
		return 0L;
	}

	ASSERT(pRequestInfo->pBuffer);

	//pRequestInfo->nPage != ::GetCurrentPage()
	if (IsInvalidPage(pRequestInfo->nPage, pRequestInfo->uBook) ||
		pRequestInfo->dwBufferSize <= 2) {	// Empty response has 2 characters \r\n
		delete[] pRequestInfo->pBuffer;
		delete pRequestInfo;
		return 0L;
	}

	EnterCriticalSection(&g_CriticalSection);

	// I'd need to use strtok but I can't because I used it 
	// in the ParsePicHeaderLine function, so I did my own
	// strtok funtion that it's even better for this case.

	char* p = (char*) pRequestInfo->pBuffer;
	char* nexttoken = p;
	PICTUREINFO pi;
	CRect rect;
	CSize totalSize(0, 0);

	GetClientRect(rect);

	/* // Debug...
	CString msg;
	msg.Format(_T("%hs"), p);
	MessageBox(msg);
	*/ //End Debug

	//m_nInterPicGap = (int)(rect.Width() * INTERPICFACTOR);
	
	rect.left = m_nLeftMargin;
	rect.top = m_nInterPicGap;
	rect.right -= m_nRightMargin;

	pi.uBook = pRequestInfo->uBook;
	pi.nPage = pRequestInfo->nPage;
	pi.uIndex = 0;

	while (*nexttoken != '\0')
	{
		if (*nexttoken == '\n' /* the token*/)
		{
			*nexttoken = '\0';
			
			pi.uIndex++;

			if (ParsePicHeaderLine(p, pi)) {
				totalSize.cy += AddPicture(pi, rect);
				totalSize.cy += m_nInterPicGap;
			}
			else 
				AddErrorPicture(pi, rect);

			p = ++nexttoken;

			if (*p == '\r' || *p == '\n')
				break;
		}
		else nexttoken++;
	}

	SetScrollSizes(MM_TEXT, totalSize);

	delete[] pRequestInfo->pBuffer;
	delete pRequestInfo;

	LeaveCriticalSection(&g_CriticalSection);

	return 0L;
}

bool CPictureView::ParsePicHeaderLine(char* szLine, PICTUREINFO& pi)
{
	char* p;

	if (!(p = strtok(szLine, ","))) return false;

	// Get name string
	pi.strName.Format(TEXT("%hs"), p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get type
	//strupr(p);
	if (strcmp(p, "DIB") == 0) pi.nType = OT_DIB;
	else if (strcmp(p, "AVI") == 0) pi.nType = OT_AVI;
	else if (strcmp(p, "WAV") == 0) pi.nType = OT_WAV;
	else if (strcmp(p, "OLE") == 0) pi.nType = OT_OLE;
	else if (strcmp(p, "JPG") == 0) pi.nType = OT_JPG;
	else if (strcmp(p, "WAV") == 0) pi.nType = OT_GIF;
	else pi.nType = OT_UNK;
	if (!(p = strtok(NULL, ","))) return false;

	// Get note status
	if (*p == 'y' || *p == 'Y') pi.bHasNote = true;
	else pi.bHasNote = false;
	if (!(p = strtok(NULL, ","))) return false;

	// Get the small version's size
	pi.dwSmallSize = atoi(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the large version's size
	pi.dwLargeSize = atoi(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the small picture version's width
	pi.lSmallWidth = atol(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the small picture version's height
	pi.lSmallHeight = atol(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the large picture version's width
	pi.lLargeWidth = atol(p);
	if (!(p = strtok(NULL, ","))) return false;

	// Get the large picture version's height
	pi.lLargeHeight = atol(p);
	
	// It's the last param and I don't care if 
	// there are more things, so I return true.

	return true;
}

int CPictureView::GetPictureHeight(int uIndex)
{
	CRect rect;
	//m_picArray[uIndex]->GetWindowRect(rect);
	return rect.Height();
}

BOOL CPictureView::DestroyWindow() 
{
	Empty();
	
	return CView::DestroyWindow();
}

LRESULT CPictureView::OnNoteClick(WPARAM wParam, LPARAM lParam /*Pic Index*/)
{
	if (!AfxGetMainWnd()->SendMessage(WM_NOTE_CLICK, wParam, lParam))
		return FALSE;
	
	if (m_pActiveNotePicWnd && 
		m_pActiveNotePicWnd != (CPictureWnd*)wParam)
	{
		m_pActiveNotePicWnd->SetNoteActive(false);
	}
	
	m_pActiveNotePicWnd = (CPictureWnd*)wParam;

	return TRUE;
}

LRESULT CPictureView::OnAddNote(WPARAM wParam, LPARAM lParam /*Pic Index*/)
{
	if (AfxGetMainWnd()->SendMessage(WM_ADD_NOTE, wParam, lParam))
	{
		if (m_pActiveNotePicWnd)
			m_pActiveNotePicWnd->SetNoteActive(false);
		
		m_pActiveNotePicWnd = (CPictureWnd*)wParam;
		return TRUE;
	}
	return FALSE;
}

LRESULT CPictureView::OnDeleteNote(WPARAM wParam, LPARAM lParam /*Pic Index*/)
{
	if (AfxGetMainWnd()->SendMessage(WM_DELETE_NOTE, 
		wParam, lParam))
	{
		if (m_pActiveNotePicWnd == (CPictureWnd*)wParam)
			m_pActiveNotePicWnd = NULL;

		return TRUE;
	}

	return FALSE;
}

LRESULT CPictureView::OnPushPicture(WPARAM wParam, LPARAM lParam)
{
	if (wParam == 0) return FALSE;

	POSITION pos = m_picList.FindIndex((int) wParam - 1);
	
	if (!pos) return FALSE;

	m_picList.GetAt(pos)->PushPicture();
	
	return TRUE;
}
